<?php
 /**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * Creates/Updates the database tables and fills them with entries (depending on 
 * selected options during setup process)
 *
 * @package    Contenido setup
 * @version    0.2.2
 * @author     unknown
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 *
 *   $Id$:
 */

if (!defined('CON_FRAMEWORK')) {
    define('CON_FRAMEWORK', true);
}
define('C_CONTENIDO_PATH', '../admin/');

include_once('lib/startup.php');


checkAndInclude(C_CONTENIDO_PATH . 'includes/functions.database.php');
checkAndInclude(C_CONTENIDO_PATH . 'classes/class.version.php');

function removeDir($sDir, $leaveDir = false) {
    if (!$dh = @opendir($sDir)) {
        return;
    }
    while (($obj = readdir($dh)) !== false) {
        if (($obj == '.') || ($obj == '..')) {
            continue;
        }
        if (!@unlink($sDir . '/' . $obj)) {
            removeDir($sDir . '/' . $obj);
        }
    }
    closedir($dh);
    if (!$leaveDir){
        @rmdir($sDir);
    }
}

if (!hasMySQLiExtension()) {
    die("Can't detect MySQLi extension");
} else {
    $cfg['database_extension'] = $_SESSION['dbmode'];
}

checkAndInclude('../conlib/prepend.php');

?>
<html>
<head></head>
<body style="color: <?php echo $_GET['c']; ?>; background-color: <?php echo $_GET['bc']; ?>;">
<?php

$db = getSetupMySQLDBConnection(false);

if (checkMySQLDatabaseCreation($db, $_SESSION['dbname'])) {
    $db = getSetupMySQLDBConnection();
}

$currentstep = intval($_GET['step']);

if ($currentstep == 0) {
    $currentstep = 1;
}

if (($_SESSION["setuptype"] == 'setup') && (strlen($_SESSION["dbencoding"])) && ($currentstep == 1)) {
    $db->query('ALTER DATABASE `' . $_SESSION['dbname'] . '` CHARACTER SET ' . substr($_SESSION['dbencoding'], 0, strpos($_SESSION['dbencoding'], '_')) . ' COLLATE ' . $_SESSION['dbencoding']);
}

// Count DB Chunks
$file = fopen('data/tables.txt', 'r');
$step = 1;
while (($data = fgetcsv($file, 4000, ';')) !== false) {
    if ($count == 50) {
        $count = 0;
        $step ++;
    }
    
    if ($currentstep == $step) {
        if ($data[7] == '1') {
            $drop = true;
        } else {
            $drop = false;
        }
        dbUpgradeTable($db, $_SESSION['dbprefix'].'_'.$data[0], $data[1], $data[2], $data[3], $data[4], $data[5], $data[6], '', $drop);
        
        if ($db->Errno != 0) {
            $_SESSION['install_failedupgradetable'] = true;
        }
    }
    
    $count ++;
    $fullcount ++;
}

$baseChunks = explode("\n", file_get_contents('data/base.txt'));
if ($_SESSION['setuptype'] == 'setup') {
    $sysadminChunk = explode("\n", file_get_contents('data/sysadmin.txt'));
    $fullChunks = array_merge($baseChunks, $sysadminChunk);
} elseif ($_SESSION['setuptype'] == 'upgrade') {
    $upgradeChunks = explode("\n", file_get_contents('data/upgrade.txt'));
    $fullChunks = array_merge($baseChunks, $upgradeChunks);
} else {
    $fullChunks = $baseChunks;
}

list($root_path, $root_http_path) = getSystemDirectories();

$totalsteps = ceil($fullcount / 50) + count($fullChunks) + 1;
foreach ($fullChunks as $fullChunk) {
    $step ++;
    if ($step == $currentstep) {
        $failedChunks = array();

        $replacements = array(
            '<!--{contenido_root}-->' => addslashes(str_replace('\\', '/', $root_path)),
            '<!--{contenido_web}-->' => addslashes(str_replace('\\', '/', $root_http_path))
        );

        injectSQL($db, $_SESSION['dbprefix'], 'data/' . $fullChunk, $replacements, $failedChunks);

        if (count($failedChunks) > 0) {
            if (@$fp = fopen('../admin/data/logs/setuplog.txt', 'a')) {
                foreach ($failedChunks as $failedChunk) {
                    fwrite($fp, sprintf("Setup was unable to execute SQL. MySQL-Error: %s, MySQL-Message: %s, SQL-Statements:\n%s", $failedChunk['errno'], $failedChunk['error'], $failedChunk['sql']) . "\n\n");
                }
                fclose($fp);
            }
            else {
                echo sprintf("Setup was unable to execute SQL. MySQL-Error: %s, MySQL-Message: %s, SQL-Statements:\n%s", $failedChunk['errno'], $failedChunk['error'], $failedChunk['sql']);
                echo '<script>alert("' . sprintf("Setup was unable to execute SQL. MySQL-Error: %s, MySQL-Message: %s, SQL-Statements:\n%s", $failedChunk['errno'], $failedChunk['error'], $failedChunk['sql']) . '");</script>';
            }
            $_SESSION['install_failedchunks'] = true;
        }
    }
}

$percent = intval((100 / $totalsteps) * ($currentstep));

echo '<script src="script/jquery-3.6.0.min.js"></script>' . "\n";
echo '<script>parent.updateProgressbar(' . $percent . ');</script>' . "\n";
if ($currentstep < $totalsteps) {
    printf('<script>window.setTimeout("nextStep()", 5); function nextStep() { window.location.href="dbupdate.php?step=%s&c=" + $("body").css("color") + "&bc=" + $("body").css("background-color"); }</script>', $currentstep + 1);
} else {
    $tables = array();
    $sql = 'SHOW TABLES';
    $db->query($sql);
    while ($db->next_record()) {
        $tables[] = $db->f(0);
    }

    foreach ($tables as $table) {
        if ($table != $_SESSION['dbprefix'].'_country_language_date') {
            dbUpdateSequence($_SESSION['dbprefix'].'_sequence', $table, $db);
        }
    }

    updateContenidoVersion($db, $_SESSION['dbprefix'].'_system_prop', C_SETUP_VERSION);
    updateSystemProperties($db, $_SESSION['dbprefix'].'_system_prop');

    if (isset($_SESSION['sysadminpass']) && $_SESSION['sysadminpass'] != '') {
        updateSysadminPassword($db, $_SESSION['dbprefix'].'_phplib_auth_user_md5', 'sysadmin');
    }

    $sql = 'DELETE FROM ' . $_SESSION['dbprefix'].'_code';
    $db->query($sql);

    // As con_code has been emptied, force code creation (on update)
    $sql = 'UPDATE ' . $_SESSION['dbprefix'] .'_cat_art SET createcode = 1';
    $db->query($sql);

    if ($_SESSION['setuptype'] == 'migration') {
        $aClients = listClients($db, $_SESSION['dbprefix'].'_clients');

        foreach ($aClients as $iIdClient => $aInfo) {
            updateClientPath($db, $_SESSION['dbprefix'].'_clients', $iIdClient, str_replace('\\', '/', $_SESSION['frontendpath'][$iIdClient]), str_replace('\\', '/', $_SESSION['htmlpath'][$iIdClient]));
        }
    }

    $_SESSION['start_compatible'] = false;

    if ($_SESSION['setuptype'] == 'upgrade') {
        $sql = 'SELECT `is_start` FROM `' . $_SESSION['dbprefix'].'_cat_art` WHERE `is_start` = 1';
        $db->query($sql);
        if ($db->next_record()) {
            $_SESSION['start_compatible'] = true;
        }

        # Update the texts in the content table to clean html (not encoded anymore)
        $aContents = array();
        $sql = 'SELECT `idcontent`, `value` FROM `' . $_SESSION['dbprefix'] . '_content` WHERE ((`value` LIKE "%\%%") OR ((`value` LIKE "%+%") AND (`value` NOT LIKE "% %")))';
        $db->query($sql);
        while ($db->next_record()) {
            $aContents[] = $db->toArray();
        }
        foreach ($aContents as $aContent) {
            $sql = 'UPDATE `' . $_SESSION['dbprefix'] . '_content`
                    SET `value` = "' . $db->escape(urldecode(str_replace('+', ' ', $aContent['value']))) . '"
                    WHERE (`idcontent`=' . $aContent['idcontent'] . ')';
            $db->query($sql);
        }

        # Update the texts in the mod_translations table to clean html (not encoded anymore)
        $aContents = array();
        $sql = 'SELECT `idmodtranslation`, `original`, `translation` FROM `' . $_SESSION['dbprefix'] . '_mod_translations` WHERE ((`original` LIKE "%\%%") OR ((`original` LIKE "%+%") AND (`original` NOT LIKE "% %")) OR (`translation` LIKE "%\%%") OR ((`translation` LIKE "%+%") AND (`translation` NOT LIKE "% %")))';
        $db->query($sql);
        while ($db->next_record()) {
            $aContents[] = $db->toArray();
        }
        foreach ($aContents as $aContent) {
            $sql = 'UPDATE `' . $_SESSION['dbprefix'] . '_mod_translations`
                    SET `original` = "' . addslashes($db->escape(urldecode(str_replace('+', ' ', $aContent['original'])))) . '",
                        `translation` = "' . addslashes($db->escape(urldecode(str_replace('+', ' ', $aContent['translation'])))) . '"
                    WHERE (`idmodtranslation`=' . $aContent['idmodtranslation'] . ')';
            $db->query($sql);
        }

        # Update the texts in the properties table to clean html (not encoded anymore)
        $aContents = array();
        $sql = 'SELECT `idproperty`, `name`, `value` FROM `' . $_SESSION['dbprefix'] . '_properties` WHERE ((`name` LIKE "%\%%") OR ((`name` LIKE "%+%") AND (`name` NOT LIKE "% %")) OR (`value` LIKE "%\%%") OR ((`value` LIKE "%+%") AND (`value` NOT LIKE "% %")))';
        $db->query($sql);
        while ($db->next_record()) {
            $aContents[] = $db->toArray();
        }
        foreach ($aContents as $aContent) {
            $sql = 'UPDATE `' . $_SESSION['dbprefix'] . '_properties`
                    SET `name` = "' . addslashes($db->escape(urldecode(str_replace('+', ' ', $aContent['name'])))) . '",
                        `value` = "' . addslashes($db->escape(urldecode(str_replace('+', ' ', $aContent['value'])))) . '"
                    WHERE (`idproperty`=' . $aContent['idproperty'] . ')';
            $db->query($sql);
        }
    }

    // Update Keys
    $aNothing = array();

    injectSQL($db, $_SESSION['dbprefix'], 'data/indexes.sql', array(), $aNothing);

    if ($_SESSION['setuptype'] == 'upgrade') {
        /**
         * For upgrades from pre 2.1.0 we need to eventually move the plugins to the new admin folder.
         * We can't just upgrade them, as they may be individually reworked.
         * Also, if the backend was in another folder, we need to redirect the users.
         */
        $sPath = '';
        if (file_exists($root_path . "/admin/data/config/config.php")) {
            $sPath = 'admin';
        }
        elseif (file_exists($root_path . "/admin/includes/config.php")) {
            $sPath = 'admin';
        }
        elseif (file_exists($root_path . "/drugcms/includes/config.php")) {
            $sPath = 'drugcms';
            checkAndInclude($root_path . "/drugcms/includes/config.php");
        }
        else {
            $sPath = 'contenido';
            checkAndInclude($root_path . "/contenido/includes/config.php");
        }
        switch ($sPath) {
            case 'drugcms':
            case 'contenido':
                
                # Copy the plugins (except system plugins) to the admin plugins folder
                checkAndInclude($root_path . '/setup/lib/functions.filesystem.php');
                $sSource = $cfg['path']['contenido'] . 'plugins/';
                $aPlugins = getSubdirs($sSource);
                foreach ($aPlugins as $sPlugin) {
                    $aPlugin = explode('/', $sPlugin);
                    $sName = $aPlugin[(count($aPlugin) - 2)];
                    if (!in_array($sName, array('chains', 'ds', 'frontendlogic', 'frontendusers', 'mom', 'pim', 'repository'))) {
                        recursive_copy($sPlugin, str_replace($sPath, 'admin', $sPlugin));
                    }
                }

                # If SEO-URL is installed, copy it's config file(s)
                foreach (glob($root_path . '/' . $sPath . '/data/config/config.mod_rewrite_*.php') as $sFile) {
                    copy($sFile, str_replace($sPath, 'admin', $sFile));
                }
                foreach (glob($root_path . '/' . $sPath . '/includes/config.mod_rewrite_*.php') as $sFile) {
                    copy($sFile, str_replace(array($sPath, '/includes/'), array('admin', '/data/config/'), $sFile));
                }
                
                # Create a new file 'index.php' in the old backend folder to redirect to the new one
                $fp = fopen($cfg['path']['contenido'] . 'index.php', 'w');
                fwrite($fp, '<?php header("Location: ../admin/index.php"); ?>');
                fclose($fp);
                break;
        }
        /**
         * Do some cleanup in the conlib directory
         */
        foreach (glob($root_path . '/conlib/*.inc') as $sFile) {
            unlink($sFile);
        }
        /**
         * For upgrades from pre 2.2.0 we must delete some old config files (they are now in admin/data/config/).
         */
        unlink($root_path . '/' . $sPath . '/includes/cfg_language_de.inc.php');
        unlink($root_path . '/' . $sPath . '/includes/cfg_sgl.inc.php');
        unlink($root_path . '/' . $sPath . '/includes/config.chains.php');
        unlink($root_path . '/' . $sPath . '/includes/config.colors.php');
        unlink($root_path . '/' . $sPath . '/includes/config.http_check.php');
        unlink($root_path . '/' . $sPath . '/includes/config.misc.php');
        unlink($root_path . '/' . $sPath . '/includes/config.path.php');
        unlink($root_path . '/' . $sPath . '/includes/config.templates.php');
        /**
         * For upgrades from pre 2.2.0 we need to eventually move module files into the new structure.
         */
        $aClients = listClients($db, $_SESSION['dbprefix'].'_clients');
        foreach ($aClients as $iIdClient => $aInfo) {
            $sPath = $aInfo['frontendpath'];
            $sModPath = $sPath . 'data/modules/';
            $aMods = getSubdirs($sModPath);
            foreach ($aMods as $sMod) {
                $sMod = basename($sMod);
                if (((file_exists($sModPath . $sMod . '/' . $sMod . '-input.php')) || (file_exists($sModPath . $sMod . '/' . $sMod . '-output.php')) || (file_exists($sModPath . $sMod . '/input.php')) || (file_exists($sModPath . $sMod . '/output.php'))) && (!is_dir($sModPath . $sMod . '/php/'))) {
                    mkdir($sModPath . $sMod . '/css/', 0755, true);
                    mkdir($sModPath . $sMod . '/images/', 0755, true);
                    mkdir($sModPath . $sMod . '/js/', 0755, true);
                    mkdir($sModPath . $sMod . '/php/', 0755, true);
                    mkdir($sModPath . $sMod . '/templates/', 0755, true);
                    if (file_exists($sModPath . $sMod . '/' . $sMod . '-input.php')) {
                        rename($sModPath . $sMod . '/' . $sMod . '-input.php', $sModPath . $sMod . '/php/' . $sMod . '-input.php');
                    }
                    elseif (file_exists($sModPath . $sMod . '/input.php')) {
                        rename($sModPath . $sMod . '/input.php', $sModPath . $sMod . '/php/' . $sMod . '-input.php');
                    }
                    if (file_exists($sModPath . $sMod . '/' . $sMod . '-output.php')) {
                        rename($sModPath . $sMod . '/' . $sMod . '-output.php', $sModPath . $sMod . '/php/' . $sMod . '-output.php');
                    }
                    elseif (file_exists($sModPath . $sMod . '/output.php')) {
                        rename($sModPath . $sMod . '/output.php', $sModPath . $sMod . '/php/' . $sMod . '-output.php');
                    }
                }
            }
        }
        /**
         * Modules not exported to files need to get so right now.
         */
        $sql = 'SELECT `idclient`, `name`, `input`, `output`
                FROM `' . $_SESSION['dbprefix'] . '_mod`
                ORDER BY `idmod`';
        $db->query($sql);
        while ($db->next_record()) {
            $sPath = $aClients[$db->f('idclient')]['frontendpath'];
            $sModPath = $sPath . 'data/modules/';
            $sMod = uplCreateFriendlyName($db->f('name'));
            if (!is_dir($sModPath . $sMod)) {
                mkdir($sModPath . $sMod . '/css/', 0755, true);
                mkdir($sModPath . $sMod . '/images/', 0755, true);
                mkdir($sModPath . $sMod . '/js/', 0755, true);
                mkdir($sModPath . $sMod . '/php/', 0755, true);
                mkdir($sModPath . $sMod . '/templates/', 0755, true);
                file_put_contents($sModPath . $sMod . '/php/' . $sMod . '-input.php', $db->f('input'));
                file_put_contents($sModPath . $sMod . '/php/' . $sMod . '-output.php', $db->f('output'));
            }
        }
        /**
         * Layouts not exported to files need to get so right now.
         */
        $sql = 'SELECT `idclient`, `name`, `code`
                FROM `' . $_SESSION['dbprefix'] . '_lay`
                ORDER BY `idlay`';
        $db->query($sql);
        while ($db->next_record()) {
            $sPath = $aClients[$db->f('idclient')]['frontendpath'];
            $sLayPath = $sPath . 'data/layouts/';
            if (!is_dir($sLayPath)) {
                mkdir($sLayPath, 0755, true);
            }
            $sLay = uplCreateFriendlyName($db->f('name'));
            if (!file_exists($sLayPath . $sLay . '.php')) {
                file_put_contents($sLayPath . $sLay . '.php', $db->f('code'));
            }
        }
        /**
         * The front_content.php might have changed, copy it from 
         * admin/external/frontend/front_content.php to each client directory
         */
        foreach ($aClients as $aClient) {
            if (!file_exists($aClient['frontendpath'] . 'front_content.php.sic')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.sic');
            }
            elseif (!file_exists($aClient['frontendpath'] . 'front_content.php.s01')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.s01');
            }
            elseif (!file_exists($aClient['frontendpath'] . 'front_content.php.s02')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.s02');
            }
            elseif (!file_exists($aClient['frontendpath'] . 'front_content.php.s03')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.s03');
            }
            elseif (!file_exists($aClient['frontendpath'] . 'front_content.php.s04')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.s04');
            }
            copy(C_CONTENIDO_PATH . 'external/frontend/front_content.php', $aClient['frontendpath'] . 'front_content.php');
        }
        /**
         * Switch $force to true for all clients that have modules_in_files switched on
         */
        foreach ($aClients as $iClient => $aClient) {
            $sql = 'SELECT `value`
                    FROM `' . $_SESSION['dbprefix'] . '_properties`
                    WHERE ((`idclient`=' . $iClient . ')
                       AND (`itemtype`="clientsetting")
                       AND (`type`="modules_in_files")
                       AND (`name`="use"))';
            $db->query($sql);
            if ($db->next_record()) {
                if (in_array($db->f('value'), array('true', '1'))) {
                    file_put_contents($aClient['frontendpath'] . 'config.php', str_replace('$force = 0;', '$force = 1;', file_get_contents($aClient['frontendpath'] . 'config.php')));
                }
            }
        }
    }
    if (($_SESSION['setuptype'] == 'upgrade') || ($_SESSION['setuptype'] == 'migration')) {
        /**
         * Correct the backend paths in each frontend config.php
         */
        foreach ($aClients as $iClient => $aClient) {
            # Explode the paths into arrays
            $aAdminPath = explode('/', $root_path);
            $aClientPath = explode('/', $aClient['frontendpath']);
            # Compare the arrays
            $n = count($aClientPath);
            for ($i = 0; $i < $n; $i ++) {
                if ($aAdminPath[$i] == $aClientPath[$i]) {
                    # If the values for the same position are equal, remove them from both arrays
                    unset($aAdminPath[$i], $aClientPath[$i]);
                }
                else {
                    # On the first different value exit from the comparison
                    break;
                }
            }
            # Remove the last array elements as they are empty (the paths ended with a slash)
            unset($aAdminPath[($n - 1)], $aClientPath[($n - 1)]);
            # Compile the relative path from the remaining arrays
            $sPath = str_repeat('../', count($aClientPath)) . implode('/', $aAdminPath) . 'admin/';
            # Get the config.php content
            $sContent = file_get_contents($aClient['frontendpath'] . 'config.php');
            # Exchange the backend path
            $p1 = strpos($sContent, '$contenido_path = "');
            if ($p1 !== false) {
                $p1 += 19;
                $p2 = strpos($sContent, '";', $p1);
                $sContent = substr($sContent, 0, $p1) . $sPath . substr($sContent, $p2);
            }
            # Write the new config.php file
            file_put_contents($aClient['frontendpath'] . 'config.php', $sContent);
        }
    }
    
    printf('<script>
        setTimeout(function() {
            setTimeout(function() {
                parent.document.getElementById("installing").style.display="none";
                parent.document.getElementById("installingdone").style.display="block";
                parent.document.getElementById("next").style.visibility="visible";
            }, 1000);
            parent.document.getElementById("Next").click();
        }, 1000);
        </script>');
}
?>
</body>
</html>