<?php
 /**
 * Project:
 * DZIRCMS Content Management System
 *
 * Description:
 * DZIRCMS setup script
 *
 * Requirements:
 * @con_php_req 7
 *
 * @package    DZIRCMS setup
 * @version    2.0.1
 * @author     unknown
 * @copyright  Spider IT Deutschland
 * @license    https://www.dzir.org/cms/license/
 * @link       https://www.dzir.org/cms/
 * @link       https://www.spider-it.de
 * 
 *
 * {@internal
 *   created  unknown
 *   modified 2008-07-07, bilal arslan, added security fix
 *   modified 2011-01-21, Ortwin Pinke, added php-errorhandling function calls, uncomment if needed
 *   modified 2011-02-24, Murat Purc, extended mysql extension detection and some other changes
 *   modified 2011-02-28, Murat Purc, normalized setup startup process and some cleanup/formatting
 *   modified 2021-07-03, René Mansveld, reworked completely for DZIRCMS
 *
 *   $Id$:
 * }}
 *
 */

if (!defined('CON_FRAMEWORK')) {
    define('CON_FRAMEWORK', true);
}

ob_start();

define('C_CONTENIDO_PATH', '../admin/');

include_once('lib/startup.php');


if (is_array($_REQUEST)) {
    foreach ($_REQUEST as $key => $value) {
        if (($value != '' && $key != 'dbpass') || ($key == 'dbpass' && $_REQUEST['dbpass_changed'] == 'true')) {
            $_SESSION[$key] = $value;
        }
    }
}


if (getPHPIniSetting('session.use_cookies') == 0) {
    $sNotInstallableReason = 'session_use_cookies';
    checkAndInclude('steps/notinstallable.php');
}

if (strlen($_SESSION['dbmode'])) {
    $cfg['database_extension'] = strtolower($_SESSION['dbmode']);
} elseif (hasMySQLiExtension()) {
    $cfg['database_extension'] = 'mysqli';
} else {
    $sNotInstallableReason = 'database_extension';
    checkAndInclude('steps/notinstallable.php');
}

checkAndInclude('../conlib/prepend.php');

i18nRegisterDomain("setup", 'locale/');
if (array_key_exists('language', $_SESSION)) {
    i18nInit('locale/', $_SESSION['language']);
}
if (phpversion() < C_SETUP_MIN_PHP_VERSION) {
    $sNotInstallableReason = 'php_version';
    checkAndInclude('steps/notinstallable.php');
}

if (array_key_exists('step', $_REQUEST)) {
    $sStep = $_REQUEST['step'];
} else {
    $sStep = 'languagechooser';
}

switch ($sStep) {
    case 'languagechooser':
        checkAndInclude('steps/languagechooser.php');
        break;
    case 'setuptype':
        checkAndInclude('steps/setuptype.php');
        break;
    case 'doinstall':
        checkAndInclude('steps/setup/doinstall.php');
        break;
    case 'doupgrade':
        checkAndInclude('steps/upgrade/doupgrade.php');
        break;
    case 'domigration':
        checkAndInclude('steps/migration/domigration.php');
        break;
    case 'makeconfig':
        checkAndInclude('makeconfig.php');
        break;
    default:
        $sType = substr($sStep, 0, -1);
        $iStep = (int) substr($sStep, -1);
        checkAndInclude('steps/' . $sType . '/step' . $iStep . '.php');
        break;
}
ob_end_flush();
?>